@@ -119,7 +119,9 @@ expandMacro x = return x
 evalElement :: Element -> State EvalState [Output]
 evalElement el
     | Elements fm es <- el        = evalElements es >>= \os ->
-                                      return [Output os fm]
+                                      if null os
+                                         then return []
+                                         else return [Output os fm]
     | Const    s   fm       <- el = return $ addSpaces s
                                            $ if fm == emptyFormatting
                                                 then [OPan (readCSLString s)]
@@ -139,8 +141,13 @@ evalElement el
                                     ifEmpty (evalNames False s n d)
                                             (withNames s el $ evalElements sub)
                                             (appendOutput fm)
-    | Substitute (e:els)    <- el = ifEmpty (consuming $ substituteWith e)
-                                            (getFirst els) id
+    | Substitute (e:els)    <- el = do
+                        res <- consuming $ substituteWith e
+                        if null res
+                           then if null els
+                                   then return [ONull]
+                                   else evalElement (Substitute els)
+                           else return res
     -- All macros and conditionals should have been expanded
     | Choose _ _  _         <- el = error $ "Unexpanded Choose"
     | Macro    s   _        <- el = error $ "Unexpanded macro " ++ s
@@ -176,7 +183,9 @@ evalElement el
                             case numVars of
                               ["number-of-volumes"] -> not $ any (== "1") nums
                               ["number-of-pages"]   -> not $ any (== "1") nums
-                              _ -> any ('-' `elem`) nums
+                              _ -> any
+                                   (\x -> '-' `elem` x || '\x2013' `elem` x)
+                                   nums
                          pluralizeTerm x = x
                      if null res
                         then return []
@@ -203,10 +212,6 @@ evalElement el
                                         , env = (env s)
                                           {names = tail $ names (env s)}}) >> return r
 
-      getFirst        [] = return []
-      getFirst    (x:xs) = whenElse ((/=) []  <$> substituteWith x)
-                                    (consuming $  substituteWith x)
-                                    (getFirst xs)
       getVariable f fm s = if isTitleVar s || isTitleShortVar s
                            then consumeVariable s >> formatTitle s f fm else
                            case map toLower s of
@@ -339,7 +344,7 @@ formatNumber f fm v n
 
 checkRange :: [CslTerm] -> String -> String
 checkRange _ [] = []
-checkRange ts (x:xs) = if x == '-'
+checkRange ts (x:xs) = if x == '-' || x == '\x2013'
                        then pageRange ts ++ checkRange ts xs
                        else x             : checkRange ts xs
 
@@ -370,7 +375,7 @@ breakNumericString :: [String] -> [String]
 breakNumericString [] = []
 breakNumericString (x:xs)
     | isTransNumber x = x : breakNumericString xs
-    | otherwise       = let (a,b) = break (flip elem "&-,") x
+    | otherwise       = let (a,b) = break (flip elem "&-\x2013,") x
                             (c,d) = if null b then ("","") else (take 1 b, tail b)
                         in filter (/= []) $  a : c : breakNumericString (d : xs)
 
@@ -384,7 +389,9 @@ formatRange fm p = do
 
       tupleRange [] = []
       tupleRange (x:"-":[]  ) = return (x,[])
+      tupleRange (x:"\x2013":[]  ) = return (x,[])
       tupleRange (x:"-":y:xs) = (x, y) : tupleRange xs
+      tupleRange (x:"\x2013":y:xs) = (x, y) : tupleRange xs
       tupleRange (x:      xs) = (x,[]) : tupleRange xs
 
       joinRange (a, []) = a
