@@ -20,7 +20,7 @@ module Text.CSL.Proc.Disamb where
 
 import Control.Arrow ( (&&&), (>>>), second )
 import Data.Char ( chr )
-import Data.List ( elemIndex, elemIndices, find, findIndex, sortBy, mapAccumL
+import Data.List ( elemIndex, find, findIndex, sortBy, mapAccumL
                  , nub, nubBy, groupBy, isPrefixOf )
 import Data.Maybe
 import Data.Ord ( comparing )
@@ -143,7 +143,7 @@ updateOName :: [NameData] -> Output -> Output
 updateOName n o
     | OName _ _ [] _ <- o = o
     | OName k x _  f <- o = case elemIndex (ND k (clean x) [] []) n of
-                              Just i -> OName [] (nameDataSolved $ n !! i) [] f
+                              Just i -> OName emptyAgent (nameDataSolved $ n !! i) [] f
                               _      -> o
     | otherwise           = o
     where
@@ -192,15 +192,15 @@ getDuplCiteData b1 b2 g
       collide    = proc rmExtras . proc rmNameHash . proc rmGivenNames . whatToGet
       citeData   = nubBy (\a b -> collide a == collide b && key a == key b) $
                    concatMap (mapGroupOutput $ getCiteData) g
-      findDupl f = filter (flip (>) 1 . length . flip elemIndices (map f citeData) . f) citeData
-      duplicates = if b2 then findDupl (collide &&& citYear)
-                         else findDupl  collide
+      duplicates = [c | c <- citeData , d <- citeData , collides c d]
+      collides x y = x /= y && (collide x == collide y)
+                            && (not b2 || citYear x == citYear y)
 
 rmExtras :: [Output] -> [Output]
 rmExtras os
-    | Output         x f : xs <- os = case rmExtras x of
+    | Output         x _ : xs <- os = case rmExtras x of
                                            [] -> rmExtras xs
-                                           ys -> Output ys f : rmExtras xs
+                                           ys -> ys ++ rmExtras xs
     | OContrib _ _ x _ _ : xs <- os = OContrib [] [] x [] [] : rmExtras xs
     | OYear        y _ f : xs <- os = OYear y [] f : rmExtras xs
     | ODel             _ : xs <- os = rmExtras xs
@@ -395,7 +395,7 @@ rmGivenNames o
 
 rmNameHash :: Output -> Output
 rmNameHash o
-    | OName _ s ss f <- o = OName [] s ss f
+    | OName _ s ss f <- o = OName emptyAgent s ss f
     | otherwise           = o
 
 -- | Add, with 'proc', a give name to the family name. Needed for
